#include <windows.h>
#include "adc.h"
#include "Coproc.h"

#define MAX_NUMBER_ADC 4
//******************************************************************************
/// Initialize HANDLE for the all four ADC.
/// @param[in]    hPort       Array of HANDLE to ADC channels
/// @retval       TRUE        Success
/// @retval       FALSE       Failure
BOOL initAllAdc(HANDLE *adc)
{
	adc[0] = Adc_Init(L"ADC1");						///< Initialize ADC1, without affecting hardware registers
	if (adc[0] == NULL)
	{
		printf("Error in Adc_Init() ADC1\r\n");
		return FALSE;
	}
	adc[1] = Adc_Init(L"ADC2");						///< Initialize ADC2, without affecting hardware registers
	if (adc[1] == NULL)
	{
		printf("Error in Adc_Init() ADC2\r\n");
		return FALSE;
	}
	adc[2] = Adc_Init(L"ADC3");						///< Initialize ADC3, without affecting hardware registers
	if (adc[2] == NULL)
	{
		printf("Error in Adc_Init() ADC3\r\n");
		return FALSE;
	}
	adc[3] = Adc_Init(L"ADC4");						///< Initialize ADC4, without affecting hardware registers
	if (adc[3] == NULL)
	{
		printf("Error in Adc_Init() ADC4\r\n");
		return FALSE;
	}
	return TRUE;
}

//******************************************************************************
/// Configure ADC with bit resolution and average sample and Open ADC
/// @param[in]    hPort       HANDLE to ADC channels
/// @param[in]    bit		  DWORD bit resolution
/// @param[in]    sample      DWORD Average sample
/// @retval       TRUE        Success
/// @retval       FALSE       Failure
BOOL configureADC(HANDLE adc, DWORD bit, DWORD sample)
{
	BOOL returnValue = TRUE;
	if(adc != NULL)
	{
		returnValue = Adc_SetConfigInt(adc, L"BitResolution", bit, StoreVolatile);///< Set a bit resolution configuration parameter.
		returnValue = Adc_SetConfigInt(adc, L"AvgSamples", sample, StoreVolatile);///< Set a average sample configuration parameter.
		returnValue = Adc_Open(adc);											  ///< Apply configuration to the hardware, If it is stored in registry path.
	}
	return returnValue;
}


//******************************************************************************
/// Read ADC channel
/// @param[in]    hPort       HANDLE to ADC channel
/// @param[out]   adcValue    DWORD ADC raw value.
/// @return       Number of Bytes received. 0 means failure.
DWORD ReadADCChannel(HANDLE adc, DWORD *adcValue)
{
	DWORD value = 0;
	DWORD noOfByte = 0;
	noOfByte =  Adc_Read(adc, &value, sizeof(value));	///< Read raw adc value
	*adcValue = value;
	return noOfByte;
}
//******************************************************************************
/// Close & De-initialize HANDLE for the ADC
/// @param[in]    hPort       HANDLE to ADC channel
void closeADC(HANDLE adc)
{
	if(adc != NULL)
	{
		Adc_Close(adc);									///< Close ADC channel
		Adc_Deinit(adc);								///< Deinit ADC channel
	}
}

int WINAPI WinMain(HINSTANCE instance, HINSTANCE prevInstance, LPWSTR cmdLine, int cmdShow)
{
	eSoc    socType;									///< To get soc type
	HANDLE	adcHwnd[4]	= {NULL};						///< Array of Handle for four adc channel
	int		adcChannel	= 0;							///< Variable used as counter in for loop
	DWORD	data[4]		= {0};							///< Variable to get data from adc read	
	DWORD	volts		= 0;							///< Variable to get adc value in  volt
    DWORD	millivolts	= 0;							///< variable to get adc value in milli volts
	CHAR	exitChar	='0';							///< Variable to get user input from console
	tVersion ver;										///< Variable to get library version

	wprintf(L"****** Toradex ADC demo V1.0 ******\n");
	Adc_GetVersion(&ver);								///< Get Library version
	wprintf(L"Toradex Library Version %d:%d:%d\r\n", ver.Major, ver.Minor, ver.Build);
	socType = Cop_GetSoc();								///< Get SOC type and print on console
	switch(socType)						
	{
	case Soc_PXA270:
		printf("Colibri PXA270 detected\n");
		printf("ADC Resolution is: 12-bits\n");
		break;
	case Soc_PXA320:
		printf("Colibri PXA320 detected\n");
		printf("ADC Resolution is: 12-bits\n");
		break;
	case Soc_PXA300:
		printf("Colibri PXA300 detected\n");
		printf("PXA300 ADC feature is not present, PXA300XT resolution is 12-bits\n");
		break;
	case Soc_PXA310:
		printf("Colibri PXA310 detected\n");
		printf("ADC Resolution is: 12-bits\n");
		break;
	case Soc_TEGRA2:
		printf("Colibri T20 detected\n");
		printf("ADC Resolution is: 12-bits\n");
		break;
	case Soc_TEGRA3:
		printf("Colibri T30 detected\n");
		printf("ADC Resolution is: 12-bits\n");
		break;
	case Soc_VF50:
		printf("Colibri VF50 detected\n");
		printf("ADC Resolution is: 12-bits\n");
		break;
	case Soc_VF61:
		printf("Colibri VF61 detected\n");
		printf("ADC Resolution is: 12-bits\n");
		break;
	case Soc_IMX6S:
		printf("Colibri iMX6S detected\n");
		printf("ADC Resolution is: 12-bits\n");
		break;
	case Soc_IMX6DL:
		printf("Colibri iMX6DL detected\n");
		printf("ADC Resolution is: 12-bits\n");
		break;
	case Soc_IMX6Q:
		printf("Apalis iMX6Q detected\n");
		printf("ADC Resolution is: 12-bits\n");
		break;
	default:
		printf("Processor type unknown\n");
		break;
	}
	initAllAdc(adcHwnd);										///< Local function to initialize all adc handle
	for(adcChannel = 0; adcChannel < MAX_NUMBER_ADC; adcChannel++)
	{
		configureADC(adcHwnd[adcChannel], 12, 8);				///< Local Function to configure adc and open adc
	}

	while(exitChar != 'e')
	{
		for (adcChannel = 0; adcChannel < MAX_NUMBER_ADC; adcChannel++)
		{
			Adc_Read(adcHwnd[adcChannel], &data[adcChannel], sizeof(DWORD));	///< Local Function Read ADC channel in loop
			volts = data[adcChannel] / 1000000;
			millivolts = (data[adcChannel] - 1000000*volts) / 1000;
			printf("Channel_%d:%d raw\t\tChannel_%d: %d.%03d V\n", adcChannel, data[adcChannel], adcChannel, volts, millivolts);		
		}

		printf("\nPress 'e' to close the program or 'enter' to rescan\n");
		scanf_s("%c", &exitChar);								///< Get input from user to exit from program
		fflush(stdin);
	}
	for (adcChannel = 0; adcChannel < MAX_NUMBER_ADC; adcChannel++)
	{
		closeADC(adcHwnd[adcChannel]);							///< Local function to close and De-init ADC handle
	}
	return(TRUE);
}
